package be.axi.planner.domain;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.drools.planner.config.SolverFactory;
import org.drools.planner.config.XmlSolverFactory;
import org.drools.planner.core.Solver;

public class MaintenancePlanning {

	public static void main(String[] args) {

		// Build the Solver
		long startTime = new Date().getTime();
		SolverFactory solverFactory = new XmlSolverFactory(
				"/plannerSolverConfig.xml");
		Solver solver = solverFactory.buildSolver();

		MaintenanceSchedule unsolvedSchedule = initiateSchedule();

		// Solve the problem
		solver.setPlanningProblem(unsolvedSchedule);
		solver.solve();

		MaintenanceSchedule solvedSchedule = (MaintenanceSchedule) solver
				.getBestSolution();
		System.out.println("isEveryProblemFactChangeProcessed: "
				+ solver.isEveryProblemFactChangeProcessed());
		System.out.println("isTerminateEarly: " + solver.isTerminateEarly());
		System.out.println("score: " + solvedSchedule.getScore().getScore());

		// Display the result
		System.out.println("\nSolved :\n" + toDisplayString(solvedSchedule));
		long endTime = new Date().getTime();
		System.out.println("Done in " + ((endTime - startTime)) + " ms.");

	}

	private static MaintenanceSchedule initiateSchedule() {
		MaintenanceSchedule unsolvedSchedule = new MaintenanceSchedule();

		List<MaintenanceJob> projects = new ArrayList<MaintenanceJob>();
		projects.add(new MaintenanceJob(new Plane("Airbus A350 - XJ34", 3)));
		projects.add(new MaintenanceJob(new Plane("Fokker R40 - HR09", 4)));
		projects.add(new MaintenanceJob(new Plane("Boeing 737 - PJ23", 3)));

		List<MaintenanceTask> tasks = new ArrayList<MaintenanceTask>();
		for (MaintenanceJob project : projects) {
			tasks.addAll(project.getTasks());
		}

		unsolvedSchedule.setMaintenanceTasks(tasks);

		// Available inspectors
		List<Technician> inspectors = new ArrayList<Technician>();
		inspectors.add(new Technician("Tom"));
		inspectors.add(new Technician("Andy"));
		inspectors.add(new Technician("Kristof"));
		inspectors.add(new Technician("Marc"));
		inspectors.add(new Technician("John"));
		inspectors.add(new Technician("Hilde"));
		inspectors.add(new Technician("Floor"));
		inspectors.add(new Technician("Arwen"));
		inspectors.add(new Technician("Jana"));
		inspectors.add(new Technician("Joan"));

		unsolvedSchedule.setTechnicians(inspectors);

		// Available periods
		List<Period> periods = new ArrayList<Period>();
		periods.add(new Period("P1", toDate("02/07/2012"), toDate("21/07/2012")));
		periods.add(new Period("P2", toDate("06/08/2012"), toDate("24/08/2012")));
		periods.add(new Period("P3", toDate("03/09/2012"), toDate("21/09/2012")));
		periods.add(new Period("P4", toDate("01/10/2012"), toDate("19/10/2012")));
		unsolvedSchedule.setPeriods(periods);

		return unsolvedSchedule;
	}

	public static String toDisplayString(MaintenanceSchedule solvedSchedule) {

		StringBuilder displayString = new StringBuilder();
		for (MaintenanceTask project : solvedSchedule.getMaintenanceTasks()) {
			displayString.append("  ").append(project.getLabel()).append("\n");
		}
		return displayString.toString();
	}

	public static Date toDate(String aDateString) {
		Date date = null;
		try {
			SimpleDateFormat formatter = new SimpleDateFormat("dd/MM/yyyy");
			date = formatter.parse(aDateString);
		} catch (ParseException e) {
			throw new RuntimeException("Could not parse date-string ("
					+ aDateString + "): " + e.getMessage(), e);
		}
		return date;
	}

}
