package be.axi.planner.domain;

import org.drools.planner.api.domain.entity.PlanningEntity;
import org.drools.planner.api.domain.variable.PlanningVariable;
import org.drools.planner.api.domain.variable.ValueRange;
import org.drools.planner.api.domain.variable.ValueRangeType;
import be.axi.planner.solution.TaskDifficultyWeightFactory;

@PlanningEntity(difficultyWeightFactoryClass = TaskDifficultyWeightFactory.class)
public class MaintenanceTask {

	private Period period;
	private MaintenanceJob job;
	private Technician technician;
	private String id;

	public MaintenanceTask(MaintenanceJob aJob, String aId) {
		this.job = aJob;
		this.id = aId;
	}

	public String getId(){
		System.out.println("Task: "+id +", period: "+period + ", technician: " + technician);
		return id;
	}
	
	@PlanningVariable
    @ValueRange(type = ValueRangeType.FROM_SOLUTION_PROPERTY, solutionProperty = "technicians")
	public Technician getTechnician() {
		return technician;
	}
	
	public void setTechnician(Technician aTechnician) {
		this.technician = aTechnician;
	}

	@PlanningVariable
    @ValueRange(type = ValueRangeType.FROM_SOLUTION_PROPERTY, solutionProperty = "periods")
	public Period getPeriod() {
		return period;
	}

	public void setPeriod(Period period) {
		this.period = period;
	}
	
	public String getPeriodId(){
		return period == null ? null : period.getId();
	}
	
    public String getJobId(){
        return job == null ? null : job.getId();
    }

	public Object getLabel() {
		String label;
		if (period == null){
			label = "Inspection of " + getPlane().getName() + " not scheduled!";
		}else {
			label = "Inspection of " + getPlane().getName() + " on " + period.toString();
		}
		if (technician != null){
			label += " by " + technician.getName();
		}
		return label;
	}

	public MaintenanceTask clone() {
		MaintenanceTask clone = new MaintenanceTask(job, id);
		clone.period = this.period;
		clone.technician = this.technician;
		return clone;
	}

	/** The difficulty of a project depends on the number of required inspectors, number of entries in loadMap...
	 * @return measurement of difficulty
	 */
	public int getDifficulty() {
		return job.getDifficulty();
	}
	
	@Override
	public String toString() {
		return "Maintenance of " + getId();
	}
	
	public boolean equals(Object o) {
        if (this == o) {
            return true;
        } else if (o instanceof MaintenanceTask) {
        	MaintenanceTask other = (MaintenanceTask) o;
            return other.getId().equals(this.getId());
        } else {
            return false;
        }
    }
	
	public Plane getPlane(){
		return job.getPlane();
	}

}
